#!/usr/bin/env node

import { colarg } from "colarg";
import * as fs from "fs";
import * as path from "path";

let cli = new colarg(process.argv.slice(2));

cli.addOption({
	name: "locale",
	alias: "l",
	description: "A specific locale to use when generating the output.",
	required: false,
	type: "string"
});

cli.addOption({
	name: "map",
	alias: "m",
	description: "A JSON file containing a map of all locales available, this will cause iteration over all locales and output into corresponding folders/files.",
	required: true,
	type: "string"
});

cli.addOption({
	name: "output",
	alias: "o",
	description: "A path to output the converted file to.",
	required: false,
	type: "string"
})

cli.addOption({
	name: "output-type",
	alias: "t",
	description: "The method to output converted files, either 'file' or 'folder', 'file' will cause the output to be written as a file into the current folder, 'folder' will generate folders with the locales as name and output the original file name into that.",
	required: false,
	defaults: "folder",
	type: "string"
})

cli.addOption({
	name: "i18n-prefixed",
	alias: "i",
	description: "Whether to prefix injected variables with 'i18n'.",
	required: false,
	defaults: true,
	type: "boolean"
})

cli.enableHelp()
const args = cli.getArgs();

// First of all, we need to get the input file, this will be the first default argument.
if (!args.default || args.default.length == 0) {
	throw new Error("Missing required argument for input file, make sure it is passed as anonymous argument.");
}

const inputFile = args.default[0];
// Check if the given path is absolute or relative, if it's relative we will have to pad it with the current working directory.
let inputContent: string;
if (fs.existsSync(inputFile)) {
	// The path is absolute, read the file.
	inputContent = fs.readFileSync(inputFile).toString();
} else {
	// Pad the path with the current working directory
	inputContent = fs.readFileSync(process.cwd() + '/' + inputFile).toString();
}

(async function() {
	// Get the map of translations.
	if (args.map) {
		// For autocompletion in editors we want to support JSON as well as JS files.
		// Let's check what we have here.
		let locales: {[key: string]: {[key: string]: string}};
		if (path.extname(args.map) == ".js") {
			locales = (await import(path.join(process.cwd(), args.map))).default
		} else if (path.extname(args.map) == ".json") {
			locales = JSON.parse(fs.readFileSync(getAbsolute(args.map)).toString())
		}

		// Check if only one locale should be used as output.
		if (args.locale) {
			translate(locales, args.locale);
		} else {
			// Otherwise we will loop through the locale map.
			for (const localeName in locales) {
				translate(locales, localeName)
			}
		}
	}
})();


function translate(locales: {[key: string]: {[key: string]: string}}, localeName: string) {
	const translations = locales[localeName];

	// We now have a list of translations, a list of key-value pairs with translation names as keys and their corresponding translation values as values.
	let translatedFile = inputContent;
	for (let wordName in translations) {
		const translation = translations[wordName];

		// Check if we need to prefix the name with 'i18n'
		if (args["i18n-prefixed"]) {
			wordName = "i18n_" + wordName;
		}
		translatedFile = translatedFile.replace(new RegExp(wordName, 'g'), `"${translation.replace(/"/g, '\\"').replace(/\n/g, '\\n')}"`);
	}
	// The file has been translated, let's write it to the output.
	// If the output-type is 'folder', we need to create multiple subfolders where we output the file name.
	let fileName = path.basename(inputFile);
	// The output argument will be used as the root folder for the output.
	const rootFolder = getAbsolute(args.output);
	if (args["output-type"] == "folder") {
		// Maybe create the output folder first.
		let outputFolder = path.join(rootFolder, localeName);
		if (!fs.existsSync(outputFolder)) {
			fs.mkdirSync(outputFolder, { recursive: true })
		}

		fs.writeFileSync(path.join(outputFolder, fileName), translatedFile);
	} else {
		fs.writeFileSync(path.join(rootFolder, localeName + "_" + fileName), translatedFile);
	}
}

function getAbsolute(p: string) {
	if (fs.existsSync(p)) {
		return p;
	} else {
		return path.join(process.cwd(), p);
	}
}